<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 2003 Administrator (admin@email.test)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is 
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/** 
 * Plugin 'spamshield' for the 'spamshield' extension.
 *
 * This modul will analyce the POST / GET variables for possible spam.
 * If spam is detectet:
 * 1)  it will redirect to a spam-error page.
 * 2)  it will present the spam-reason to the spam-error page for further processing
 *      ... error message on page
 *      ... info mail to administrator
 *      ... log entry in DB-table
 * If no spam is dectect:
 * 1) if fields have been renamed the original names are restored
 *
 * @author        Ronald Steiner <ronald.steiner@googlemail.com>
 *                initial testing with fe hook
 */

require_once (PATH_tslib."class.tslib_pibase.php");
require_once (PATH_tslib."class.tslib_content.php");

class tx_spamshield_pi3 extends tslib_pibase  {
        
	var $prefixId = "tx_spamshield_pi3";                // Same as class name
	var $scriptRelPath = "pi3/class.tx_spamshield_pi3.php";        // Path to this script relative to the extension dir.
	var $extKey = "spamshield";        // The extension key.
	
	var $conf;      		// extension configurations-Array
	var $GETparams;    		// GET variables
	var $POSTparams;    	// POST variables
	var $GPparams;    		// POST variables
	var $spamTarget = 'neverspam.html'; // target if spam is submitted
	var $spamReason;     		// description of the error
	var $spamWeight = 0;    // weight of the spam


	function main (&$GETparams, &$POSTparams, &$GPparams, &$conf) {
        if ($conf['enable'] == 1) {
			// initialize
			$this->init($GETparams,$POSTparams,$GPparams,$conf);
			// check if no data from Form has been submitted
   			if (!$this->formDataSubmit()) {
				return; // nothing to do
			}
			# environment checks
			if ($this->conf['useragent']) {
	            $this->useragent();
			}
			if ($this->conf['referer']) {
	            $this->referer();
			}
			# session and cookies
	        if ($this->conf['session']) {
	            $this->session();
			}
	        if ($this->conf['cookie']) {
	            $this->cookie();
			}
	        if ($this->conf['jsCookie']) {
	            $this->jsCookie();
			}
			# rename back to original names
			if ($this->conf['rename']) {
        		$this->rename();
	        }
			# check form data for spam input
			if ($this->conf['javascript']) {
        		$this->javascript();
	        }
			if ($this->conf['honeypot']) {
        		$this->honeypot();
	        }
			if ($this->conf['unique']) {
	            $this->unique();
			}
			# finished
            $this->spamReason = ltrim($this->spamReason,',');
			// if spam detected => log and redirect (better to log here, as some spam bots won't follow the redirect)
        	if ($this->spamWeight >=  $this->conf['criticalSpamWeight']) {
        	    if ($this->conf['dbLog']) {
        	        $this->dbLog();
				}
            	header("HTTP/1.1 302 Moved temporarily");
				header("Location: ".t3lib_div::getIndpEnv('TYPO3_REQUEST_DIR').$this->spamTarget."?tx_spamshield_pi1[spamReason]=".urlencode($this->spamReason));
				die ();
				# maybe add redirect url: '&post='.serialize($this->POSTparams).'&get='serialize($this->GETparams) ... what ever
				# => this would make a more eloquent error page possible by: unserialize($this->piVars['post'])
			}
		}
	}

	#####################################################
	## Functions for initalizing                    #####
	#####################################################
	/**
	 * initialize
	 * set class variables
	 *
	 * @param	array       POST params
	 * @param   array       configuration array
	 * @return  nothing
	 */
	function init(&$GETparams,&$POSTparams,&$GPparams,&$conf) {
		// set $this->conf
		$this->conf =& $conf;
	    if (!$this->conf['dbLog.']['pid']) {
	        $this->conf['dbLog.']['pid'] = 1;
		}
		// set GET / POST / GP vars
		$this->GETparams =& $GETparams;
		$this->POSTparams =& $POSTparams;
		$this->GPparams =& $GPparams;
		// set spamTarget
		if ($conf['spamTarget']) {
        	$local_cObj = t3lib_div::makeInstance('tslib_cObj');
        	$this->spamTarget = $local_cObj->stdWrap($conf['spamTarget'],$conf['spamTarget.']);
		}
	}

	/**
	 * Put a log entry in the DB if spam is detected
	 *
	 * @param	nothing
	 * @return  boolean
	 */
	function dbLog() {
        $db_values = array (
        	'pid' => $this->conf['dbLog.']['pid'],
            'tstamp' => time(),
            'crdate' => time(),
            'spamWeight' => $this->spamWeight,
            'spamReason' => $this->spamReason,
            'postvalues' => t3lib_div::view_array($this->POSTparams),
            'getvalues' => t3lib_div::view_array($this->GETparams),
            'pageid' => $GLOBALS['TSFE']->id,
            'ip' => t3lib_div::getIndpEnv('REMOTE_ADDR'),
            'useragent' => t3lib_div::getIndpEnv('HTTP_USER_AGENT'),
            'referer' => t3lib_div::getIndpEnv('HTTP_REFERER')
        );
        $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_spamshield_log', $db_values); // DB entry
    }
	#####################################################

	/**
	 * checking if a form data has been submitted
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function formDataSubmit() {
	    if (is_array($this->POSTparams) && sizeof($this->POSTparams) > 0) {
	        return true;  	// form data has submitted via POST or GET
		}
		if ($this->conf['detectForm'] && $this->conf['detectForm.']['typicalVars']) {
			$typicalFormVars = explode(',',$this->conf['detectForm.']['typicalVars']);
			foreach ($typicalFormVars as $getVar) {
		    	if ($this->GPparams[$getVar]) {
		        	return true; // typical form var exists
				}
			}
		}
		return false;       // no form data in POST or GET
	}
	
	/**
	* useragant
	*
    * Every browser sends a HTTP_USER_AGENT value to a server.
    * 	So a missing HTTP_USER_AGENT value almost always indicates a spammer bot.
    */
	function useragent() {
	    #if (getenv("HTTP_USER_AGENT") == "") {
	    if (t3lib_div::getIndpEnv('HTTP_USER_AGENT') == "") {
		   	$this->spamReason .= ",useragent";
		   	$this->spamWeight += $this->conf['useragent.']['weight'];
		   	return;  		// = spam
		}
		return; // no spam;
	}

	/**
	* referer
	*
    * The most of browsers (all modern browsers) send a HTTP_REFERER value,
    * 	... which would contain the submitted form URL.
    *   ... which therefore should be from same domain.
    *  Whereas clever bots send this value, a missing HTTP_REFERER value could mean a bot submitting.
    *  Note. There are several firewall and “security” products which block HTTP_REFERER by default.
    *  So, none of these people could send a message if you block posting without HTTP_REFERER.
	 */
	function referer() {
	    #if (getenv("HTTP_REFERER") == "") {
	    if (t3lib_div::getIndpEnv('HTTP_REFERER') == "") {
		   	$this->spamReason .= ",referer";
		   	$this->spamWeight += $this->conf['referer.']['weight'];
		   	return;  		// = spam
		}
		return; // no spam;
	}

	/**
	 * checks if session is availabel
	 * some bots send spam without loading the pabe
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function session() {
	    // get session timestamp
		$sess_tstamp = $GLOBALS['TSFE']->fe_user->getKey('ses','spamshield_tstamp');
		if (!$sess_tstamp) {
		   	$this->spamReason .= ",session.errormsg";
		   	$this->spamWeight += $this->conf['session.']['weight'];
		   	return;  		// = spam (form data with no session)
		}
		if ($sess_tstamp && $this->conf['session.']['time']) {
		    list($minTime, $maxTime) = explode('-',$this->conf['session.']['time']);
		    if ($sess_tstamp + $maxTime < time()) {
				$this->spamReason .= ",session.errormsgSlow";
				$this->spamWeight += $this->conf['session.']['weight'];
				return; 	// = spam (very slow form submission)
			}
			if ($sess_tstamp + $minTime > time()) {
				$this->spamReason .= ",session.errormsgFast";
				$this->spamWeight += $this->conf['session.']['weight'];
				return; 	// = spam (very fast form submission)
			}
		}
		return;      // = no spam
	}

	/**
	 * checks if cookie is availabel
	 * some bots don't accept cookies normally
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function cookie() {
	    // get session timestamp
		$cookie_tstamp = $_COOKIE['spamshield_tstamp'];
		if (!$cookie_tstamp) {
		   	$this->spamReason .= ",cookie.errormsg";
		   	$this->spamWeight += $this->conf['cookie.']['weight'];
		   	return;  		// = spam (form data with no session)
		}
		if ($cookie_tstamp && $this->conf['cookie.']['time']) {
		    list($minTime, $maxTime) = explode('-',$this->conf['cookie.']['time']);
		    if ($cookie_tstamp + $maxTime < time()) {
				$this->spamReason .= ",cookie.errormsgSlow";
				$this->spamWeight += $this->conf['cookie.']['weight'];
				return; 	// = spam (very slow form submission)
			}
			if ($cookie_tstamp + $minTime > time()) {
				$this->spamReason .= ",cookie.errormsgFast";
				$this->spamWeight += $this->conf['cookie.']['weight'];
				return; 	// = spam (very fast form submission)
			}
		}
		return;      // = no spam
	}

	/**
	 * checks if cookie is availabel
	 * some bots don't accept cookies normally
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function jsCookie() {
	    // get session timestamp
		$cookie_tstamp = $_COOKIE['spamshield_tstamp_js'];
		if (!$cookie_tstamp) {
		   	$this->spamReason .= ",jsCookie.errormsg";
		   	$this->spamWeight += $this->conf['jsCookie.']['weight'];
		   	return;  		// = spam (form data with no session)
		}
		if ($cookie_tstamp && $this->conf['jsCookie.']['time']) {
		    list($minTime, $maxTime) = explode('-',$this->conf['jsCookie.']['time']);
		    if ($cookie_tstamp + $maxTime < time()) {
				$this->spamReason .= ",jsCookie.errormsgFast";
				$this->spamWeight += $this->conf['jsCookie.']['weight'];
				return; 	// = spam (very slow form submission)
			}
			if ($cookie_tstamp + $minTime > time()) {
				$this->spamReason .= ",jsCookie.errormsgFast";
				$this->spamWeight += $this->conf['jsCookie.']['weight'];
				return; 	// = spam (very fast form submission)
			}
		}
		return;      // = no spam
	}

	/**
	 * renaming of the submited data
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function rename() {
		if ($this->POSTparams['rename']) {
		    $targetArray =& $this->POSTparams;
		}
		elseif ($this->GETparams['rename']) {
		    $targetArray =& $this->GETparams;
		}
		else {
		    return;
		}
		$renameArray = unserialize(str_replace('###','"',$targetArray['rename']));
		if (!is_array($renameArray)) {
		    return;
		}
		# <input type="image" name="... empty / no array ..." will be send twice
		foreach ($targetArray as $key => $val) {
			if (strstr($key,'_')) {
				list($newKey,$waste) = explode('_',$key);
				$targetArray[$newKey] = $val;
			}
		}
		foreach ($renameArray as $cryptName => $orgName) {
			$orgName = str_replace(']','',$orgName);
			$orgNameParts = explode('[',$orgName);
			$a = array();
			if (trim($targetArray[$cryptName]) != "") {
				$this->setDepth($a,$orgNameParts,$targetArray[$cryptName]);
				$targetArray = array_merge_recursive($targetArray, $a);
			}
            unset($targetArray[$cryptName]);
		}
		unset($targetArray['rename']);
		$this->GPparams = array_merge_recursive($this->GETparams,$this->POSTparams);
	}
	/* help function for above */
	function setDepth(&$a, $parts,$finalVal) {
	    if (strstr('[',$string)) {
			$parts = explode('[',$string);
		}
		$thisPart = array_shift($parts);
    	if (sizeof($parts) >0) {
    	    $this->setDepth($a[$thisPart], $parts, $finalVal);
    	}
    	else {
            $a[$thisPart] = $finalVal;
		}
	}
	
	/**
	 * checks if the hidden field is changed by javascript onclick
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function javascript () {
	 	if ($this->GPparams[$this->conf['javascript.']['fieldname']] != $this->conf['javascript.']['true']) {
	 	    $this->spamReason .= ",javascript.errormsg";
	 	    $this->spamWeight += $this->conf['javascript.']['weight'];
	    	return; // = spam (javascript is not enabled)
		}
		else {
		    return;  // = no spam
		}
	}
	
	/**
	 * checks the post variables if a honey pot has been filled.
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function honeypot() {
	    $fields = explode(',',$this->conf['honeypot.']['fields']);
		foreach ($fields as $field) {
	        if ($this->GPparams[$field]) {
	        	$this->spamReason .= ",honeypot.errormsg";
	        	$this->spamWeight += $this->conf['honeypot.']['weight'];
		    	return true;  // = spam (fields are filled)
	        }
	    }
		return false;  // = no spam (fields not filled)
	}

	/**
	 * checks the GP vars for double content - bots often use e.g.: location = name
	 *
	 * @param	nothing
	 * @return  nothing
	 */
	function unique () {
	    // flat the GPparams array
	    $flatGPparamsString = $this->implode_r(",",$this->GPparams);
	    $flatGPparamsArray = explode(",",$flatGPparamsString);
		// filter empty values and integers
		$flatGPparamsArrayFiltered = array();
		for ($i = 0; $i < sizeof($flatGPparamsArray); $i++) {
		    if ($flatGPparamsArray[$i] != "" && ! is_numeric($flatGPparamsArray[$i])) {
            	$flatGPparamsArrayFiltered[] = $flatGPparamsArray[$i];
			}
		}
		if ($flatGPparamsArrayFiltered != array_unique($flatGPparamsArrayFiltered)) {
		    $this->spamReason .= ",unique.errormsg";
		    $this->spamWeight += $this->conf['unique.']['weight'];
		    return; // = spam (same input in multiple fields)
		}
		else {
		    return; // = no spam
		}
	}
	/* help function for above - implode recursive a multidimensional array */
	function implode_r ($glue, $pieces){
 		$out = "";
 		foreach ($pieces as $piece) {
  			if (is_array ($piece)) {
			  	$out .= $this->implode_r ($glue, $piece); // recurse
			}
			else {
				$out .= $glue.$piece;
			}
		}
		$out = ltrim($out, $glue);
 		return $out;
 	}

  
}

if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/spamshield/pi3/class.tx_spamshield_pi3.php"]){
        include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/spamshield/pi3/class.tx_spamshield_pi3.php"]);
}

?>